"""The storyboard package.

Modules:
frame -- extract video frames
metadata -- extract video metadata and generate string for pretty-printing
storyboard -- generate video storyboards with metadata reports

Console scripts:
metadata -- CLI interface to the metadata module
storyboard -- CLI interface to the storyboard module
"""
